@@ -1,3 +1,9 @@
+# numerically stable evaluation of
+# 1 - exp(-x)^2
+dexp2 <- function(x,Exp=exp(-x)) { ifelse(Exp<0.7071068,1-Exp^2,2*Exp*sinh(x)) }
+# 1 - exp(-x)^1
+dexp1 <- function(x,Exp=exp(-x)) { ifelse(Exp<0.5,1-Exp,2*sqrt(Exp)*sinh(x/2)) }
+
 ###############################
 # Propagator/Green's function and Two-time correlation from Langevin equation for Kalman filter and simulations
 # random CTMM objects need to be run through get.taus() first, to precompute various parameters
@@ -7,64 +13,140 @@ langevin <- function(dt,CTMM)
   tau <- CTMM$tau
   sigma <- methods::getDataPart(CTMM$sigma)
 
-  if(K<=1 && (length(tau)==0 || tau[1]==0))
+  if(K<=1) # IID-BM-OU
   {
+    # IID limit
     Green <- array(0,c(1,1))
     Sigma <- array(1,c(1,1))
-  }
-  else if(K==1)
-  {
-    c0 <- exp(-dt/tau)
-    Green <- array(c0,c(1,1))
-    Sigma <- array(1-c0^2,c(1,1))
-  }
-  else if(K==2)
+
+    if(K)
+    {
+      if(tau[1]==Inf) # BM
+      {
+        if(dt<Inf) { Green[1,1] <- 1 }
+        # absorbing 1/tau into sigma # VAR -> Diffusion
+        Sigma[1,1] <- 2*dt
+      }
+      else if(dt<Inf) # (BM,OU,IID]
+      {
+        dtau <- dt/tau
+        c0 <- exp(-dtau)
+        Green[1,1] <- c0
+        Sigma[1,1] <- dexp2(dtau,Exp=c0)
+      }
+    } # >IID
+  } # IID-BM-OU
+  else if(K==2) # IOU-OUF-OUO
   {
     Omega2 <- CTMM$Omega2
+    #IID limit
+    Green <- rbind( c(0,0) , c(0,0) )
+    Sigma <- rbind( c(1,0) , c(0,Omega2) )
 
-    if(dt==Inf) # make this numerically relative in future
-    {
-      Green <- rbind( c(0,0) , c(0,0) )
-      Sigma <- rbind( c(1,0) , c(0,Omega2) )
-    }
-    else # finite time matrices
-    {
-      TT <- CTMM$TfOmega2
-      f <- CTMM$f.nu[1] # mean(f)
-      nu <- CTMM$f.nu[2] # nu || omega
+    f <- CTMM$f.nu[1] # mean(f)
+    nu <- CTMM$f.nu[2] # nu || omega
+    TT <- CTMM$TfOmega2 # 2 f / Omega^2
+    fdt <- f*dt
 
-      # function representation choice
-      if(tau[1]>tau[2] && nu*dt>0.8813736) # exponential functions
+    if(tau[1]==Inf) # IOU
+    {
+      dtau <- dt/tau[2]
+      Exp <- exp(-dtau)
+      DExp <- dexp1(dtau,Exp) # 1-exp(dt/tau[2]/2)^2 # 1-exp(dt/tau[2])
+
+      if(dt<Inf)
       {
-        Exp <- exp(-dt/tau)/diff(tau)
+        Green[1,1] <- 1
+        Green[1,2] <- tau[2]*DExp
+        Green[2,2] <- Exp
+      }
+
+      # absorbing one 1/tau[1] into each sigma & Omega^2
+      c12 <- DExp^2 # (1-exp(dt/tau[2]/2)^2)^2 # modulo two powers of 1/tau[1]
+      Sigma[1,1] <- 2*dt - 2*tau[2]*DExp - c12/Omega2
+      if(dt<Inf) { Sigma[c(2,3)] <- TT*c12 } # 0 at dt=Inf
+      Sigma[2,2] <- Omega2 * dexp2(dtau,Exp)
+    }
+    else if(dt<Inf) # (IOU,OUF/OUO,IID]
+    {
+      # function representation choice
+      nudt <- nu*dt
+      EXP <- (tau[1]>tau[2] && nudt>0.8813736)
+      if(EXP) # exponential functions
+      {
+        dtau <- dt/tau
+        dift <- diff(tau)
+        Exp0 <- exp(-dtau)
+        Exp <- Exp0/dift
         c0 <- diff(Exp*tau)
         c1 <- -diff(Exp)
         c2 <- diff(Exp/tau)
       }
       else # trigonometric and hyperbolic-trigonometric functions
       {
-        Exp <- exp(-f*dt)
+        Exp <- exp(-fdt)
 
         if(tau[1]>tau[2]) # hyperbolic-trigonometric
         {
-          SincE <- sinch(nu*dt)*Exp
-          CosE <- cosh(nu*dt)*Exp
+          Sin0 <- sinh(nudt)
+          Sinc0 <- sinch(nudt,Sin0)
+          Cos0 <- cosh(nudt)
         }
         else # trigonometric
         {
-          SincE <- sinc(nu*dt)*Exp
-          CosE <- cos(nu*dt)*Exp
+          Sin0 <- sin(nudt)
+          Sinc0 <- sinc(nudt,Sin0)
+          Cos0 <- cos(nudt)
         }
+        SincE <- Sinc0*Exp
+        CosE <- Cos0*Exp
 
-        c0 <- CosE + (f*dt)*SincE
+        c0 <- CosE + fdt*SincE
         c1 <- -(Omega2*dt)*SincE
-        c2 <- -Omega2*(CosE - (f*dt)*SincE)
+        c2 <- -Omega2*(CosE - fdt*SincE)
       } # end function representation
 
-      Green <- rbind( c(c0,-c1/Omega2) , c(c1,-c2/Omega2) )
-      Sigma <- -TT*c1^2  #off-diagonal term
-      Sigma <- rbind( c(1,0)-c(c0^2+c1^2/Omega2,Sigma) , c(0,Omega2)-c(Sigma,c1^2+c2^2/Omega2) )
-    } # end finite time matrices
+      Green[1,1] <- c0
+      Green[2,1] <- c1
+      Green[1,2] <- -c1/Omega2
+      Green[2,2] <- -c2/Omega2
+
+      # initially canceling terms
+      if(EXP)
+      {
+        dift2 <- dift^2
+        T2 <- tau^2
+        S1 <- dexp2(dtau[1],Exp0[1])
+        S2 <- dexp2(dtau[2],Exp0[2])
+        S12 <- 2*tau[1]*tau[2]*dexp1(fdt,Exp0[1]*Exp0[2])
+        Sigma[1,1] <- (T2[1]*S1 - S12 + T2[2]*S2)/dift2
+        Sigma[2,2] <- (T2[2]*S1 - S12 + T2[1]*S2)/dift2 * Omega2
+      }
+      else
+      {
+        CROSS <- fdt*Sinc0*Exp
+        OUTER <- Cos0^2*dexp2(fdt,Exp) - CROSS^2
+        CROSS <- 2*Cos0*Exp*CROSS
+        Sin2 <- Sin0^2
+
+        if(tau[1]>tau[2])
+        {
+          Sigma[1,1] <- OUTER - Sin2 - CROSS
+          Sigma[2,2] <- (OUTER - Sin2 + CROSS) * Omega2
+        }
+        else
+        {
+          Sigma[1,1] <- OUTER + Sin2 - CROSS
+          Sigma[2,2] <- (OUTER + Sin2 + CROSS) * Omega2
+        }
+      }
+
+      # initially vanishing terms
+      c12 <- c1^2
+      Sigma[1,1] <- Sigma[1,1] - c12/Omega2
+      Sigma[c(2,3)] <- TT*c12
+      Sigma[2,2] <- Sigma[2,2] - c12
+    } # end OUF/OUO
   }
 
   # fix the dimension of the filter
@@ -73,10 +155,16 @@ langevin <- function(dt,CTMM)
   { Sigma <- sigma * Sigma }
   else # 2D filter
   {
+    K <- max(1,K)
+
     Sigma <- outer(Sigma,sigma) # (k,k,d,d)
     Sigma <- aperm(Sigma,c(1,3,2,4)) # (k,k,d,d) -> (k,d,k,d)
     dim(Sigma) <- c(K*DIM,K*DIM)
 
+    # BM/IOU prior fix
+    NAN <- is.nan(Sigma)
+    if(any(NAN)) { Sigma[NAN] <- 0 }
+
     Green <- outer(Green,diag(DIM)) # (k,k,d,d)
     Green <- aperm(Green,c(1,3,2,4)) # (k,d,k,d)
     dim(Green) <- c(K*DIM,K*DIM)
@@ -85,6 +173,15 @@ langevin <- function(dt,CTMM)
   return(list(Green=Green, Sigma=Sigma))
 }
 
+
+########################
+nant <- function(x,to)
+{
+  NAN <- is.nan(x)
+  if(any(NAN)) { x[NAN] <- to }
+  return(x)
+}
+
 #############################################################
 # Internal Kalman filter/smoother for multiple derivatives, dimensions, trends
 # Kalman filter/smoother for matrix P operator and multiple mean functions
@@ -173,15 +270,17 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
       zRes[i,,] <- zRes[i,,] - (t(P) %*% zFor[i,,]) # (OBS*DIM,VEC) # zRes is initially just z
 
       Gain <- sForP %*% PDsolve(sRes[i,,]) # (K*DIM,OBS*DIM) # updated to invert Inf uncertainty to 0
+      Gain <- nant(Gain,1) # 0/0 NaN have Gain of 1
 
       # concurrent estimates
       zCon[i,,] <- zFor[i,,] + (Gain %*% zRes[i,,]) # (K*DIM,VEC)
       # manifestly positive-definite form (Joseph)
       JOSEPH <- IdH - (Gain %*% t(P)) # (K*DIM,K*DIM)
-      sCon[i,,] <- (JOSEPH %*% sFor[i,,] %*% t(JOSEPH)) # (K*DIM,K*DIM)
-      DIAG <- diag(cbind(error[i,,])) # diag() throws an error if you try to secure nrow/ncol
-      if(DIAG<Inf && DIAG>0) { sCon[i,,] <- sCon[i,,] + (Gain %*% error[i,,] %*% t(Gain)) } # don't attempt 0*Inf*0
       # this is supposed to be more stable numerically
+      sCon[i,,] <- nant(JOSEPH %*% sFor[i,,],0) %*% t(JOSEPH) + nant(Gain %*% error[i,,] %*% t(Gain),0) # (K*DIM,K*DIM) # 0^2/0 NaN have variance of 0
+      # sCon[i,,] <- nant(sCon[i,,],0) # 0^2/0 NaN have variance of 0
+      # DIAG <- diag(cbind(error[i,,])) # diag() throws an error if you try to secure nrow/ncol
+      # if(DIAG<Inf && DIAG>0) { sCon[i,,] <- sCon[i,,] + (Gain %*% error[i,,] %*% t(Gain)) } # don't attempt 0*Inf*0
 
       # update forcast estimates for next iteration
       if(i<n)
@@ -195,6 +294,11 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
     # returned likelihood & profiled mean or residuals
     if(!smooth && !sample)
     {
+      # !range
+      # remove first location from dataset
+      # this is mu[1,] with covariance error[1,,]
+      # then how does this propagate along other mu terms?
+
       # special code for DIM-1 circle - rotate (u,0) x-trend -> (0,u) y-trend and pretend like we ran y-trend through the Kalman filter
       if(DIM==1 && CTMM$circle)
       {
@@ -265,6 +369,13 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
 
       mu <- iW %*% D # (MEAN,DATA)
       # if DIM==1, dim(mu)==c(M,2), else dim(mu)==c(2*M,1) - reversed order
+      # fix for BM/IOU conditioning off first location (not really a model parameter)
+      if(!CTMM$range && length(mu))
+      {
+        # overwrite NaN stationary mean value with first location (maximizes likelihood)
+        mu[1,] <- zRes[1,DATA]
+        # remove stationary mean terms from COV
+      }
 
       # separate data from trend
       u <- zRes[,M.MEAN,drop=FALSE] # (n*OBS*DIM,MEAN)
@@ -284,6 +395,7 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
         zRes <- vapply(1:n,function(i){sRes[,,i] %*% zRes[i,,]},array(0,c(OBS*DIM,length(DATA)))) # (OBS*DIM,length(DATA),n)
         dim(zRes) <- c(OBS*DIM*length(DATA),n) # flatten
         zRes <- t(zRes) # (n,length(M.DATA))
+        if(!CTMM$range) { zRes <- zRes[-1,,drop=FALSE] }
         return(zRes)
       }
 
@@ -308,10 +420,14 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
       dim(zisRes) <- c(length(DATA),n*OBS*DIM)
       # match on right for space-time trace
       dim(zRes) <- c(n*OBS*DIM,length(DATA))
-      sigma <- (zisRes %*% zRes)/(n*DIM)
+
+      sigma <- (zisRes %*% zRes)/(n*DIM) # REML fixes this exactly for BM/IOU
 
       # log det autocorrelation matrix == trace log autocorrelation matrix
-      logdet <- mean(log(apply(sRes,1,det))) # this is 1/n times the full term
+      if(CTMM$range) # this is 1/n times the full term
+      { logdet <- mean(log(apply(sRes,1,det))) }
+      else # this is 1/(n-1) times the full term, after first is dropped
+      { logdet <- mean(log(apply(sRes,1,det)[-1])) }
 
       return(list(mu=mu,W=W,iW=iW,sigma=sigma,logdet=logdet))
     }
@@ -346,7 +462,7 @@ kalman <- function(z,u,dt,CTMM,error=NULL,smooth=FALSE,sample=FALSE,residual=FAL
     # upgrade concurrent estimates to Kriged estimates (covariance only)
     for(i in (n-1):1)
     {
-      L[i,,] <- sCon[i,,] %*% t(Green[i+1,,]) %*% PDsolve(sFor[i+1,,]) # (K*DIM,K*DIM)
+      L[i,,] <- sCon[i,,] %*% t(Green[i+1,,]) %*% PDsolve(sFor[i+1,,],pseudo=TRUE) # (K*DIM,K*DIM)
       # manifestly symmetric backsolver
       # sCon[i,,] <- sCon[i,,] + L %*% (sCon[i+1,,]-sFor[i+1,,]) %*% t(L)
       # manifestly PSD backsolver
