@@ -25,40 +25,56 @@ get.IC <- function(CTMM,IC="AICc")
   return(IC)
 }
 
-
 ##################
 # function to simplify complexity of models
 simplify.ctmm <- function(M,par)
 {
-  if("eccentricity" %in% par)
+  if("minor" %in% par)
   {
     M$isotropic <- TRUE
     M$sigma <- covm(M$sigma,isotropic=TRUE,axes=M$axes)
-    if("COV" %in% names(M)) { M$COV <- rm.name(M$COV,"eccentricity") }
+    par <- c(par,'angle')
   }
 
-  if("circle" %in% par) { M$circle <- FALSE }
+  if("major" %in% par)
+  {
+    M$isotropic <- TRUE
+    M$sigma <- covm(0,isotropic=TRUE,axes=M$axes)
+    M$tau <- NULL
+    par <- c(par,c('minor','angle','circle','tau position','tau velocity','tau','omega'))
+  }
+
+  if("circle" %in% par)
+  { M$circle <- FALSE }
 
   if("range" %in% par)
   {
+    M$sigma <- scale.covm(M$sigma,1/M$tau) # convert to diffusion matrix
     M$tau[1] <- Inf
     M$range <- FALSE
+
+    par <- c('tau','tau position')
   }
 
   # autocorrelation timescales can't be distinguished
   if("diff.tau" %in% par)
   {
-    M$tau <- c(1,1)/mean(1/M$tau)
+    M$tau <- c(1,1)*mean(M$tau)
     M$omega <- FALSE
+
+    par <- c('tau position','tau velocity')
+    M$features <- c(M$features,'tau')
   }
 
+  M$features <- M$features[M$features %nin% par]
+
   return(M)
 }
 
 
 ###############
 # keep removing uncertain parameters until AIC stops improving
-ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position",trace=FALSE,...)
+ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position",trace=FALSE,cores=1,...)
 {
   IC <- match.arg(IC,c("AICc","AIC","BIC",NA))
   MSPE <- match.arg(MSPE,c("position","velocity",NA))
@@ -108,7 +124,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
     # fit every model
     if(trace && length(GUESS)) { message("* Fitting models ",paste(names(GUESS),collapse=", "),".") }
     #? should I run select here instead of fit ?
-    GUESS <- lapply(GUESS,function(g){ctmm.fit(data,g,trace=trace2,...)})
+    GUESS <- plapply(GUESS,function(g){ctmm.fit(data,g,trace=trace2,...)},cores=cores)
 
     MODELS <<- c(MODELS,GUESS)
 
@@ -134,7 +150,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
   # all of the features we need to fit numerically
   FEATURES <- id.parameters(CTMM,UERE=UERE)$NAMES
   # consider only features unnecessary "compatibility"
-  FEATURES <- FEATURES[!(FEATURES=="area")]
+  FEATURES <- FEATURES[!(FEATURES=="major")]
   FEATURES <- FEATURES[!(FEATURES=="error")]
   FEATURES <- FEATURES[!grepl("tau",FEATURES)]
   FEATURES <- FEATURES[!(FEATURES=="omega")]
@@ -154,7 +170,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
     MLE <- get.mle()
 
     # consider non-zero eccentricity
-    if(("eccentricity" %in% FEATURES) && MLE$isotropic)
+    if(("minor" %in% FEATURES) && MLE$isotropic)
     {
       GUESS <- c(GUESS,list(MLE))
       n <- length(GUESS)
@@ -231,7 +247,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
     }
 
     # can autocorrelation timescales be distinguished?
-    if(length(CTMM$tau)==2 && (CTMM$tau[1]!=CTMM$tau[2] || CTMM$omega))
+    if(length(CTMM$tau)==2 && CTMM$tau[1]<Inf && (CTMM$tau[1]!=CTMM$tau[2] || CTMM$omega))
     {
       TEMP <- get.taus(CTMM,zeroes=TRUE)
       nu <- TEMP$f.nu[2] # frequency/difference
@@ -240,7 +256,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
       Q <- c(J %*% CTMM$COV[Q,Q] %*% J) # variance of nu
       Q <- ci.tau(nu,Q,alpha=beta)[1]
 
-      if(Q<=0 || is.na(IC))
+      if(Q<=0 || level==1 || is.na(IC))
       { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"diff.tau"))) }
     }
     else if(length(CTMM$tau)==2) # try other side if boundary if choosen model is critically damped
@@ -257,6 +273,8 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
       TEMP$omega <- sqrt(.Machine$double.eps)
       GUESS <- c(GUESS,list(TEMP))
     }
+    else if(length(CTMM$tau)==1 && level==1) # OU -> OUf (bimodal likelihood)
+    { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"diff.tau"))) }
 
     # consider if there is no circulation
     if(CTMM$circle)
@@ -268,11 +286,17 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
     # consider if eccentricity is zero
     if(!CTMM$isotropic)
     {
-      Q <- "eccentricity"
-      Q <- stats::qnorm(beta/2,mean=CTMM$sigma@par[Q],sd=sqrt(CTMM$COV[Q,Q]))
-      if(Q<=0 || is.na(IC)) { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"eccentricity"))) }
+      Q <- c("major","minor")
+      GRAD <- c(1/CTMM$sigma@par[1],-1/CTMM$sigma@par[2])
+      SD <- ifelse(all(Q %in% CTMM$features),sqrt(c(GRAD %*% CTMM$COV[Q,Q] %*% GRAD)),Inf) # variance could collapse early
+      Q <- stats::qnorm(beta/2,mean=log(CTMM$sigma@par[1]/CTMM$sigma@par[2]),sd=SD)
+      if(Q<=0 || is.na(IC)) { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"minor"))) }
     }
 
+    # is the animal even moving?
+    if(!CTMM$sigma@par['major'] && CTMM$error)
+    { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"major"))) }
+
     # consider if we can relax range residence (non-likelihood comparison only)
     if(CTMM$range && is.na(IC))
     { GUESS <- c(GUESS,list(simplify.ctmm(MLE,"range"))) }
@@ -290,7 +314,7 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
     MODELS <- sort.ctmm(MODELS,IC=IC,MSPE=MSPE)
 
     # remove redundant models
-    NAMES <- sapply(MODELS,name.ctmm)
+    NAMES <- sapply(MODELS,name.ctmm) -> names(MODELS)
     KEEP <- c(TRUE, NAMES[-1]!=NAMES[-length(NAMES)] )
     MODELS <- MODELS[KEEP]
 
@@ -303,6 +327,8 @@ ctmm.select <- function(data,CTMM,verbose=FALSE,level=1,IC="AICc",MSPE="position
 ################
 name.ctmm <- function(CTMM,whole=TRUE)
 {
+  FEATURES <- CTMM$features
+
   # base model
   tau <- CTMM$tau
   if(length(tau)==2)
@@ -315,7 +341,12 @@ name.ctmm <- function(CTMM,whole=TRUE)
   else if(length(tau)==1)
   { if(tau[1]<Inf) { NAME <- "OU" } else { NAME <- "BM" } }
   else if(length(tau)==0)
-  { NAME <- "IID" }
+  {
+    if(CTMM$sigma@par['major'] || "major" %in% FEATURES)
+    { NAME <- "IID" }
+    else
+    { NAME <- "inactive" }
+  }
 
   # isotropy
   if(CTMM$isotropic)
@@ -324,11 +355,11 @@ name.ctmm <- function(CTMM,whole=TRUE)
   { NAME <- c(NAME,"anisotropic") }
 
   # circulation
-  if(CTMM$circle)
+  if(CTMM$circle || "circle" %in% FEATURES)
   { NAME <- c(NAME,"circulation") }
 
   # error
-  if(CTMM$error)
+  if(CTMM$error || "error" %in% FEATURES)
   { NAME <- c(NAME,"error") }
 
   # mean
@@ -349,7 +380,7 @@ name.ctmm <- function(CTMM,whole=TRUE)
 }
 
 ########
-sort.ctmm <- function(x,decreasing=FALSE,IC="AICc",MSPE="position",flatten=TRUE,...)
+sort.ctmm <- function(x,decreasing=FALSE,IC="AICc",MSPE="position",flatten=TRUE,INF=FALSE,...)
 {
   if(is.na(MSPE))
   { ICS <- sapply(x,function(m){get.IC(m,IC)}) }
@@ -406,6 +437,14 @@ sort.ctmm <- function(x,decreasing=FALSE,IC="AICc",MSPE="position",flatten=TRUE,
   IND <- sort(ICS,index.return=TRUE,decreasing=decreasing)$ix
   y <- y[IND]
 
+  # BM/IOU log-likelihood is infinitely lower than OU/OUF log-likelihood
+  RANGE <- sapply(y,function(Y){Y[[1]]$range})
+  if(!is.na(IC) && any(RANGE) && any(!RANGE))
+  {
+    if(INF) { for(i in which(!RANGE)) { for(j in 1:length(y[[i]])) { y[[i]][[j]][[IC]] <- Inf } } }
+    y <- c( y[RANGE] , y[!RANGE] )
+  }
+
   if(flatten) { y <- do.call(c,y) }
 
   return(y)
@@ -423,7 +462,7 @@ summary.ctmm.list <- function(object, IC="AICc", MSPE="position", units=TRUE, ..
   MSPE <- match.arg(MSPE,c("position","velocity",NA))
 
   N <- length(object)
-  object <- sort.ctmm(object,IC=IC,MSPE=MSPE,flatten=FALSE)
+  object <- sort.ctmm(object,IC=IC,MSPE=MSPE,flatten=FALSE,INF=TRUE)
   M <- length(object)
 
   # if(N==M) { MSPE <- NA } # don't need to sort MSPE
@@ -447,8 +486,9 @@ summary.ctmm.list <- function(object, IC="AICc", MSPE="position", units=TRUE, ..
 
     # convert to meters/kilometers
     CNAME <- paste0("\u0394","RMSPE")
+    MIN <- which.min(MSPES)
     MSPES <- sqrt(MSPES)
-    MSPES <- MSPES - MSPES[1]
+    if(MSPES[1]<Inf) { MSPES <- MSPES - MSPES[MIN] }
     MIN <- min(c(abs(MSPES[MSPES!=0]),Inf))
     UNIT <- unit(MIN,if(MSPE=="position"){"length"}else{"speed"},concise=TRUE,SI=!units)
     MSPES <- MSPES/UNIT$scale
@@ -461,12 +501,26 @@ summary.ctmm.list <- function(object, IC="AICc", MSPE="position", units=TRUE, ..
   ICS <- cbind(ICS,MSPES)
   rownames(ICS) <- names(object)
 
-  DOF <- sapply(object,DOF.mean)
+  if(is.na(MSPE))
+  {
+    DOF <- sapply(object,DOF.mean)
+    DOF <- cbind(DOF)
+    colnames(DOF) <- "DOF[mean]"
+  }
+  else if(MSPE=="position")
+  {
+    DOF <- sapply(object,DOF.area)
+    DOF <- cbind(DOF)
+    colnames(DOF) <- "DOF[area]"
+  }
+  else if(MSPE=="velocity")
+  {
+    DOF <- sapply(object,DOF.speed)
+    DOF <- cbind(DOF)
+    colnames(DOF) <- "DOF[speed]"
+  }
+
   METH <- sapply(object,function(m){m$method})
-
-  DOF <- cbind(DOF)
-  colnames(DOF) <- "DOF[mean]"
-
   if(FALSE) # only prints correctly in unicode locale (Windows R bug)
   {
     DOF <- data.frame(DOF,METH)
