@@ -32,7 +32,12 @@ speed.ctmm <- function(object,data=NULL,level=0.95,robust=FALSE,units=TRUE,prior
     { CI <- summary(object,level=level,units=FALSE)$CI['speed (meters/second)',] * sqrt(pi/2/2) }
     else # elliptical velocity distribution
     {
-      NAMES <- id.parameters(object,profile=FALSE)$NAMES
+      UERE <- ifelse(object$error && "error" %nin% dimnames(object)[[1]],3,1) # propagate error uncertainty
+      STUFF <- id.parameters(object,profile=FALSE,linear=FALSE,UERE=UERE)
+      NAMES <- STUFF$NAMES
+      parscale <- STUFF$parscale
+      lower <- STUFF$lower
+      upper <- STUFF$upper
 
       fn <- function(p)
       {
@@ -45,7 +50,8 @@ speed.ctmm <- function(object,data=NULL,level=0.95,robust=FALSE,units=TRUE,prior
 
       if(prior && fast)
       {
-        GRAD <- numDeriv::grad(fn,PAR) # don't step outside of (0,1)
+        GRAD <- genD(par=PAR,fn=fn,lower=lower,upper=upper,parscale=parscale,Richardson=2,mc.cores=1)$grad
+        # GRAD <- numDeriv::grad(fn,PAR) # don't step outside of (0,1)
         VAR <- c(GRAD %*% object$COV %*% GRAD)
 
         # propagate errors chi -> chi^2
@@ -129,11 +135,14 @@ speed.ctmm <- function(object,data=NULL,level=0.95,robust=FALSE,units=TRUE,prior
           Q1 <- vint(SPEEDS,(N+1-sqrt(N))/2)
           Q2 <- vint(SPEEDS,(N+1+sqrt(N))/2)
           ERROR <- max(AVE-Q1,Q2-AVE) / AVE
+
           # correct for Inf AVE
-          if(is.nan(ERROR))
+          if(is.nan(ERROR)) { ERROR <- Inf }
+
+          if(N>1/error^2)
           {
-            ERROR <- Inf
-            warning("Speeds accumulating on boundary and expectation value may not converge.")
+            warning("Expectation values did not converge after ",N," iterations.")
+            break
           }
         }
 
@@ -170,6 +179,7 @@ speed.ctmm <- function(object,data=NULL,level=0.95,robust=FALSE,units=TRUE,prior
   CI <- rbind(CI)/UNITS$scale
   rownames(CI) <- paste0("speed (",UNITS$name,")")
   #attr(CI,"DOF") <- DOF
+  if(CI[1]==Inf) { CI[1] <- 0 } # sampled all Inf
   return(CI)
 }
 
@@ -198,7 +208,8 @@ speed.rand <- function(CTMM,data=NULL,prior=TRUE,fast=TRUE,cor.min=0.5,dt.max=NU
     # check for rare cases in sampling disribution where motion is almost but not fractal relative to sampling schedule
     # first check if non-stationary mean is irrelevant
     drift <- get(CTMM$mean)
-    if(drift@speed(CTMM)$EST/(sum(diag(CTMM$sigma))/prod(CTMM$tau))<error^2)
+    MSV <- sum(diag(CTMM$sigma))/ ifelse(CTMM$range,prod(CTMM$tau),CTMM$tau[2])
+    if(drift@speed(CTMM)$EST/MSV<error^2)
     {
       # now check if there are many steps per sampled interval
       FRAC <- CTMM$tau[2]/DT
@@ -237,18 +248,14 @@ speed.rand <- function(CTMM,data=NULL,prior=TRUE,fast=TRUE,cor.min=0.5,dt.max=NU
 # only for stationary processes
 speed.deterministic <- function(CTMM,sigma=CTMM$sigma)
 {
-  sigma <- attr(sigma,"par")
-  sigma['area'] <- sigma['area'] / prod(CTMM$tau)
+  sigma <- eigenvalues.covm(sigma)
 
-  if(CTMM$isotropic || !sigma['eccentricity'])
-  {
-    v <- sqrt(sigma['area'] * pi/2)
-  }
+  if(CTMM$isotropic || sigma[1]==sigma[2])
+  { v <- sqrt(sigma[1] * pi/2) }
   else
-  {
-    # eigen values of velocity variance
-    sigma <- sigma['area'] * exp(c(1,-1)/2*sigma['eccentricity'])
-    v <- sqrt(2/pi) * sqrt(sigma[1]) * pracma::ellipke(1-sigma[2]/sigma[1])$e
-  }
+  { v <- sqrt(2/pi) * sqrt(sigma[1]) * pracma::ellipke(1-clamp(sigma[2]/sigma[1]))$e }
+
+  v <- v/sqrt(ifelse(CTMM$range,prod(CTMM$tau),CTMM$tau[2]))
+
   return(v)
 }
